@@ -24,7 +24,8 @@
 #' a table name for SQLite and for PostgreSQL)
 #'
 #' @param value The data to be created in the database:
-#' a single data.frame, a JSON string or a list
+#' a single data.frame, a JSON string or a list;
+#' or the file name or URL of NDJSON documents
 #'
 #' @param ... Passed to functions:
 #' - CouchDB: [sofa::db_bulk_create()]
@@ -60,6 +61,8 @@ docdb_create.src_couchdb <- function(src, key, value, ...) {
   result <- try(sofa::db_create(src$con, dbname = key, delifexists = FALSE), silent = TRUE)
   if (inherits(result, "try-error") && grepl("already exists", result)) existsMessage(key)
 
+  # convert into target list
+
   # data frame
   if (class(value) == "data.frame") {
     if (is.na(match("_id", names(value)))) {
@@ -76,7 +79,18 @@ docdb_create.src_couchdb <- function(src, key, value, ...) {
 
   # convert JSON string to list
   if (class(value) == "character") {
-    value <- jsonify::from_json(value, simplify = FALSE)
+    if ((length(value)  == 1L) &&
+        (isUrl(value) || isFile(value))) {
+      # read ndjson file or url (does not work with jsonify)
+      if (isFile(value)) {
+        value <- jsonlite::stream_in(con = file(value), simplifyVector = FALSE, verbose = FALSE)
+      } else if (isUrl(value)) {
+        value <- jsonlite::stream_in(con = url(value), simplifyVector = FALSE, verbose = FALSE)
+      }
+    } else {
+      # read json string
+      value <- jsonify::from_json(value, simplify = FALSE)
+    }
   }
 
   # mangle lists
@@ -146,7 +160,21 @@ docdb_create.src_elastic <- function(src, key, value, ...) {
 
   # convert JSON string to list
   if (class(value) == "character") {
-    value <- jsonify::from_json(value, simplify = FALSE)
+
+    # since cocument IDs cannot be passed in
+    # with files, create list from file
+
+    # convert ndjson file or json string to list
+    if ((length(value)  == 1L) &&
+        (isUrl(value) || isFile(value))) {
+      if (isFile(value)) {
+        value <- jsonlite::stream_in(con = file(value), simplifyVector = FALSE, verbose = FALSE)
+      } else if (isUrl(value)) {
+        value <- jsonlite::stream_in(con = url(value), simplifyVector = FALSE, verbose = FALSE)
+      }
+    } else {
+      value <- jsonify::from_json(value, simplify = FALSE)
+    }
   }
 
   # mangle lists
@@ -201,60 +229,86 @@ docdb_create.src_mongo <- function(src, key, value, ...) {
   chkSrcMongo(src, key)
   if (docdb_exists(src, key, value, ...)) existsMessage(key)
 
-  # since mongolite does not accept valid JSON strings
-  # that come as an one-element array, convert to list
-  if (class(value) == "character") {
-    value <- jsonify::from_json(value, simplify = FALSE)
-  }
+  # directly import ndjson file
+  if ((class(value) == "character") &&
+      (length(value)  == 1L) &&
+      (isUrl(value) || isFile(value))) {
 
-  # mongolite uses _id columns in dataframes as object _id's:
-  if (class(value) == "data.frame") {
-    if (is.na(match("_id", names(value)))) {
-      # if no _id column
-      if (!identical(rownames(value),
-                     as.character(seq_len(nrow(value))))) {
-        # use rownames as _id's and remove rownames
-        value[["_id"]] <- row.names(value)
-        row.names(value) <- NULL
-      } else {
-        # add canonical _id's
-        value[["_id"]] <- uuid::UUIDgenerate(use.time = TRUE, n = nrow(value))
+    # turn into connection
+    if (isFile(value)) {
+      value <- file(value)
+    } else if (isUrl(value)) {
+      value <- url(value)
+    }
+
+    # import
+    result <- try(
+      suppressWarnings(
+        src$con$import(
+          # Stream import data in jsonlines format from a
+          # connection, similar to the mongoimport utility.
+          con = value
+          )),
+      silent = TRUE)
+
+  } else {
+
+    # since mongolite does not accept valid JSON strings
+    # that come as an one-element array, convert to list
+    if (class(value) == "character") {
+      value <- jsonify::from_json(value, simplify = FALSE)
+    }
+
+    # mongolite uses _id columns in dataframes as object _id's:
+    if (class(value) == "data.frame") {
+      if (is.na(match("_id", names(value)))) {
+        # if no _id column
+        if (!identical(rownames(value),
+                       as.character(seq_len(nrow(value))))) {
+          # use rownames as _id's and remove rownames
+          value[["_id"]] <- row.names(value)
+          row.names(value) <- NULL
+        } else {
+          # add canonical _id's
+          value[["_id"]] <- uuid::UUIDgenerate(use.time = TRUE, n = nrow(value))
+        }
+      } # if no _id column
+      # early return if no data
+      if (!nrow(value)) return(0L)
+      # convert to ndjson which is the format in which errors are raised
+      # if names (keys) had problematic characters such as . or $
+      value <- strsplit(jsonify::to_ndjson(value, unbox = TRUE), split = "\n")[[1]]
+    } # if data.frame
+
+    # convert lists (incl. from previous step) to NDJSON
+    if (class(value) == "list") {
+      # add canonical _id's
+      if ((!is.null(names(value)) && !any(names(value) == "_id")) &&
+          !any(sapply(value, function(i) any(names(i) == "_id")))
+      ) {
+        value <- lapply(value, function(i) c(
+          "_id" = uuid::UUIDgenerate(use.time = TRUE), i))
+      }
+      # split into vector of ndjson records
+      value <- jsonify::to_ndjson(value, unbox = TRUE)
+      if (any(grepl("\\}\n\\{", value))) {
+        value <- strsplit(value, split = "[}]\n[{]")[[1]]
+        value <- sub("^([^{])", "{\\1", sub("([^}])$", "\\1}", value))
       }
-    } # if no _id column
-    # early return if no data
-    if (!nrow(value)) return(0L)
-    # convert to ndjson which is the format in which errors are raised
-    # if names (keys) had problematic characters such as . or $
-    value <- strsplit(jsonify::to_ndjson(value, unbox = TRUE), split = "\n")[[1]]
-  } # if data.frame
-
-  # convert lists (incl. from previous step) to NDJSON
-  if (class(value) == "list") {
-    # add canonical _id's
-    if ((!is.null(names(value)) && !any(names(value) == "_id")) &&
-        !any(sapply(value, function(i) any(names(i) == "_id")))
-    ) {
-      value <- lapply(value, function(i) c(
-        "_id" = uuid::UUIDgenerate(use.time = TRUE), i))
     }
-    # split into vector of ndjson records
-    value <- jsonify::to_ndjson(value, unbox = TRUE)
-    if (any(grepl("\\}\n\\{", value))) {
-      value <- strsplit(value, split = "[}]\n[{]")[[1]]
-      value <- sub("^([^{])", "{\\1", sub("([^}])$", "\\1}", value))
-    }
-  }
 
-  # insert data.frame or JSON
-  result <- try(
-    suppressWarnings(
-      src$con$insert(
-        # loading either a dataframe or
-        # vector of NDJSON (!) strings
-        data = value,
-        auto_unbox = TRUE,
-        digits = NA, ...))[["nInserted"]],
-    silent = TRUE)
+    # insert data.frame or JSON
+    result <- try(
+      suppressWarnings(
+        src$con$insert(
+          # loading either a dataframe or
+          # vector of NDJSON (!) strings
+          data = value,
+          auto_unbox = TRUE,
+          digits = NA, ...))[["nInserted"]],
+      silent = TRUE)
+
+  } # if file access directly import ndjson file
 
   # generate user info
   if (inherits(result, "try-error") ||
@@ -308,8 +362,17 @@ docdb_create.src_sqlite <- function(src, key, value, ...) {
       class(value) == "json") {
     # target is data frame for next section
 
-    # convert
-    value <- jsonlite::fromJSON(value)
+    # convert ndjson file or json string to data frame
+    if ((length(value)  == 1L) &&
+        (isUrl(value) || isFile(value))) {
+      if (isFile(value)) {
+        value <- jsonlite::stream_in(con = file(value), verbose = FALSE)
+      } else if (isUrl(value)) {
+        value <- jsonlite::stream_in(con = url(value), verbose = FALSE)
+      }
+    } else {
+      value <- jsonlite::fromJSON(value)
+    }
 
     # process if value remained a list
     if (class(value) == "list") {
@@ -432,8 +495,17 @@ docdb_create.src_postgres <- function(src, key, value, ...) {
       class(value) == "json") {
     # target is data frame for next section
 
-    # convert
-    value <- jsonlite::fromJSON(value)
+    # convert ndjson file or json string to data frame
+    if ((length(value)  == 1L) &&
+        (isUrl(value) || isFile(value))) {
+      if (isFile(value)) {
+        value <- jsonlite::stream_in(con = file(value), verbose = FALSE)
+      } else if (isUrl(value)) {
+        value <- jsonlite::stream_in(con = url(value), verbose = FALSE)
+      }
+    } else {
+      value <- jsonlite::fromJSON(value)
+    }
 
     # process if value remained a list
     if (class(value) == "list") {
@@ -523,3 +595,14 @@ docdb_create.src_postgres <- function(src, key, value, ...) {
 existsMessage <- function(k) {
   message(paste0("Note: container '", k, "' already exists"))
 }
+
+isUrl <- function(x) {
+  # check if x is may be an url
+  return(grepl("^https?://", x))
+}
+
+isFile <- function(x) {
+  # check if x is the name of a readable file
+  out <- try(file.access(x, mode = 4) == 0L, silent = TRUE)
+  return(!inherits(out, "try-error") && out)
+}
