@@ -7,6 +7,7 @@
 #' - RSQLite: [DBI::dbListTables()]
 #' - Elasticsearch: [elastic::index_exists()]
 #' - CouchDB: [sofa::db_info()]
+#' - PostgreSQL: [DBI::dbListTables()]
 #'
 #' @return (logical) `TRUE` or `FALSE` to indicate
 #'  existence of container \code{key} in database
@@ -53,3 +54,9 @@ docdb_exists.src_mongo <- function(src, key, ...) {
 docdb_exists.src_sqlite <- function(src, key, ...) {
   return(any(key == DBI::dbListTables(src$con, ...)))
 }
+
+
+#' @export
+docdb_exists.src_postgres <- function(src, key, ...) {
+  return(any(key == DBI::dbListTables(src$con, ...)))
+}
